---
title: redis_streams
type: input
status: stable
categories: ["Services"]
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/input/redis_streams.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';


Pulls messages from Redis (v5.0+) streams with the XREADGROUP command. The
`client_id` should be unique for each consumer of a group.


<Tabs defaultValue="common" values={[
  { label: 'Common', value: 'common', },
  { label: 'Advanced', value: 'advanced', },
]}>

<TabItem value="common">

```yaml
# Common config fields, showing default values
input:
  label: ""
  redis_streams:
    url: tcp://localhost:6379
    body_key: body
    streams:
      - benthos_stream
    limit: 10
    client_id: benthos_consumer
    consumer_group: benthos_group
```

</TabItem>
<TabItem value="advanced">

```yaml
# All config fields, showing default values
input:
  label: ""
  redis_streams:
    url: tcp://localhost:6379
    kind: simple
    master: ""
    tls:
      enabled: false
      skip_cert_verify: false
      enable_renegotiation: false
      root_cas_file: ""
      client_certs: []
    body_key: body
    streams:
      - benthos_stream
    limit: 10
    client_id: benthos_consumer
    consumer_group: benthos_group
    start_from_oldest: true
    commit_period: 1s
    timeout: 1s
```

</TabItem>
</Tabs>

Redis stream entries are key/value pairs, as such it is necessary to specify the
key that contains the body of the message. All other keys/value pairs are saved
as metadata fields.

## Fields

### `url`

The URL of the target Redis server. Database is optional and is supplied as the URL path. `tcp` scheme is the same as `redis`


Type: `string`  
Default: `"tcp://localhost:6379"`  

```yaml
# Examples

url: :6397

url: localhost:6397

url: redis://localhost:6379

url: redis://localhost:6379/1

url: redis://localhost:6379/1,redis://localhost:6380/1
```

### `kind`

Specifies a simple, cluster-aware, or failover-aware redis client.


Type: `string`  
Default: `"simple"`  

```yaml
# Examples

kind: simple

kind: cluster

kind: failover
```

### `master`

Name of the redis master when `kind` is `failover`


Type: `string`  
Default: `""`  

```yaml
# Examples

master: mymaster
```

### `tls`

Custom TLS settings can be used to override system defaults.


Type: `object`  

### `tls.enabled`

Whether custom TLS settings are enabled.


Type: `bool`  
Default: `false`  

### `tls.skip_cert_verify`

Whether to skip server side certificate verification.


Type: `bool`  
Default: `false`  

### `tls.enable_renegotiation`

Whether to allow the remote server to repeatedly request renegotiation. Enable this option if you're seeing the error message `local error: tls: no renegotiation`.


Type: `bool`  
Default: `false`  
Requires version 3.45.0 or newer  

### `tls.root_cas_file`

An optional path of a root certificate authority file to use. This is a file, often with a .pem extension, containing a certificate chain from the parent trusted root certificate, to possible intermediate signing certificates, to the host certificate.


Type: `string`  
Default: `""`  

```yaml
# Examples

root_cas_file: ./root_cas.pem
```

### `tls.client_certs`

A list of client certificates to use. For each certificate either the fields `cert` and `key`, or `cert_file` and `key_file` should be specified, but not both.


Type: `array`  
Default: `[]`  

```yaml
# Examples

client_certs:
  - cert: foo
    key: bar

client_certs:
  - cert_file: ./example.pem
    key_file: ./example.key
```

### `tls.client_certs[].cert`

A plain text certificate to use.


Type: `string`  
Default: `""`  

### `tls.client_certs[].key`

A plain text certificate key to use.


Type: `string`  
Default: `""`  

### `tls.client_certs[].cert_file`

The path to a certificate to use.


Type: `string`  
Default: `""`  

### `tls.client_certs[].key_file`

The path of a certificate key to use.


Type: `string`  
Default: `""`  

### `body_key`

The field key to extract the raw message from. All other keys will be stored in the message as metadata.


Type: `string`  
Default: `"body"`  

### `streams`

A list of streams to consume from.


Type: `array`  
Default: `["benthos_stream"]`  

### `limit`

The maximum number of messages to consume from a single request.


Type: `int`  
Default: `10`  

### `client_id`

An identifier for the client connection.


Type: `string`  
Default: `"benthos_consumer"`  

### `consumer_group`

An identifier for the consumer group of the stream.


Type: `string`  
Default: `"benthos_group"`  

### `start_from_oldest`

If an offset is not found for a stream, determines whether to consume from the oldest available offset, otherwise messages are consumed from the latest offset.


Type: `bool`  
Default: `true`  

### `commit_period`

The period of time between each commit of the current offset. Offsets are always committed during shutdown.


Type: `string`  
Default: `"1s"`  

### `timeout`

The length of time to poll for new messages before reattempting.


Type: `string`  
Default: `"1s"`  


