---
title: lambda
type: processor
status: deprecated
categories: ["Integration"]
---

<!--
     THIS FILE IS AUTOGENERATED!

     To make changes please edit the contents of:
     lib/processor/lambda.go
-->

import Tabs from '@theme/Tabs';
import TabItem from '@theme/TabItem';

:::warning DEPRECATED
This component is deprecated and will be removed in the next major version release. Please consider moving onto [alternative components](#alternatives).
:::

Invokes an AWS lambda for each message. The contents of the message is the
payload of the request, and the result of the invocation will become the new
contents of the message.


<Tabs defaultValue="common" values={[
  { label: 'Common', value: 'common', },
  { label: 'Advanced', value: 'advanced', },
]}>

<TabItem value="common">

```yaml
# Common config fields, showing default values
label: ""
lambda:
  parallel: false
  function: ""
  region: eu-west-1
```

</TabItem>
<TabItem value="advanced">

```yaml
# All config fields, showing default values
label: ""
lambda:
  parallel: false
  function: ""
  rate_limit: ""
  region: eu-west-1
  endpoint: ""
  credentials:
    profile: ""
    id: ""
    secret: ""
    token: ""
    role: ""
    role_external_id: ""
  timeout: 5s
  retries: 3
```

</TabItem>
</Tabs>

## Alternatives

This processor has been renamed to [`aws_lambda`](/docs/components/processors/aws_lambda).

It is possible to perform requests per message of a batch in parallel by setting
the `parallel` flag to `true`. The `rate_limit`
field can be used to specify a rate limit [resource](/docs/components/rate_limits/about)
to cap the rate of requests across parallel components service wide.

In order to map or encode the payload to a specific request body, and map the
response back into the original payload instead of replacing it entirely, you
can use the [`branch` processor](/docs/components/processors/branch).

### Error Handling

When all retry attempts for a message are exhausted the processor cancels the
attempt. These failed messages will continue through the pipeline unchanged, but
can be dropped or placed in a dead letter queue according to your config, you
can read about these patterns [here](/docs/configuration/error_handling).

### Credentials

By default Benthos will use a shared credentials file when connecting to AWS
services. It's also possible to set them explicitly at the component level,
allowing you to transfer data across accounts. You can find out more
[in this document](/docs/guides/aws).

## Examples

<Tabs defaultValue="Branched Invoke" values={[
{ label: 'Branched Invoke', value: 'Branched Invoke', },
]}>

<TabItem value="Branched Invoke">


This example uses a [`branch` processor](/docs/components/processors/branch/) to map a new payload for triggering a lambda function with an ID and username from the original message, and the result of the lambda is discarded, meaning the original message is unchanged.

```yaml
pipeline:
  processors:
    - branch:
        request_map: '{"id":this.doc.id,"username":this.user.name}'
        processors:
          - lambda:
              function: trigger_user_update
```

</TabItem>
</Tabs>

## Fields

### `parallel`

Whether messages of a batch should be dispatched in parallel.


Type: `bool`  
Default: `false`  

### `function`

The function to invoke.


Type: `string`  
Default: `""`  

### `rate_limit`

An optional [`rate_limit`](/docs/components/rate_limits/about) to throttle invocations by.


Type: `string`  
Default: `""`  

### `region`

The AWS region to target.


Type: `string`  
Default: `"eu-west-1"`  

### `endpoint`

Allows you to specify a custom endpoint for the AWS API.


Type: `string`  
Default: `""`  

### `credentials`

Optional manual configuration of AWS credentials to use. More information can be found [in this document](/docs/guides/aws).


Type: `object`  

### `credentials.profile`

A profile from `~/.aws/credentials` to use.


Type: `string`  
Default: `""`  

### `credentials.id`

The ID of credentials to use.


Type: `string`  
Default: `""`  

### `credentials.secret`

The secret for the credentials being used.


Type: `string`  
Default: `""`  

### `credentials.token`

The token for the credentials being used, required when using short term credentials.


Type: `string`  
Default: `""`  

### `credentials.role`

A role ARN to assume.


Type: `string`  
Default: `""`  

### `credentials.role_external_id`

An external ID to provide when assuming a role.


Type: `string`  
Default: `""`  

### `timeout`

The maximum period of time to wait before abandoning an invocation.


Type: `string`  
Default: `"5s"`  

### `retries`

The maximum number of retry attempts for each message.


Type: `int`  
Default: `3`  


